from .letter_counter import LetterCounter


class IndexOfCoincidence:

    def __init__(self, text):
        if len(text) < 2:
            raise ValueError("text shorter than 2 characters, the incidence of coincidence can't be calculated")
        self.text = text
        self.ic = None

    def calculate(self):
        if self.ic is None:
            counter = LetterCounter(text=self.text)
            alphabet = counter.count()
            ic = 0
            for letter in alphabet:
                ic += alphabet[letter] / len(self.text) * (alphabet[letter] - 1) / (len(self.text) - 1)
            self.ic = ic
        return self.ic



from .letter_frequency_by_language import letter_frequency_by_language
from .bigram_frequency_by_language import bigram_frequency_by_language
from .index_of_coincidence_by_language import index_of_coincidence_by_language
from .frequent_short_words_by_language import frequent_short_words_by_language
from .trigram_frequency_by_language import trigram_frequency_by_language



from ..utils import string_utils


class NgramCounter:
    def __init__(self, nvalue=2, text="", joined=True):
        self.text = text
        self.nvalue = int(nvalue)
        self.joined = joined
        self.counter = {}

    def __repr__(self):
        return 'NgramCounter(nvalue={nvalue}, text={text}, joined={joined})'.format(
            nvalue=self.nvalue,
            text=self.text,
            joined=self.joined
        )

    def __str__(self):
        counter = self.count()
        return '{}'.format({", ".join(": ".join([str(c), str(counter[c])]) for c in counter)})

    def count(self):
        if self.counter == {}:
            if self.joined:
                text = "".join(self.text.split())
            else:
                text = self.text
            for position, character in enumerate(text[:-self.nvalue + 1]):
                if not string_utils.has_whitespace(text[position:position + self.nvalue]):
                    if text[position:position + self.nvalue].lower() not in self.counter.keys():
                        self.counter[text[position:position + self.nvalue].lower()] = 1
                    else:
                        self.counter[text[position:position + self.nvalue].lower()] += 1
        return self.counter

    def set_text(self, text):
        self.text = text
        self.counter = {}

    def get_text(self):
        return self.text



frequent_short_words_by_language = {
    "en": {
        1: ["a", "i"],
        2: ["am", "an", "as", "at", "be", "by", "do", "go", "he", "if", "in", "is", "it", "me", "mr", "my", "no", "of",
            "oh", "ok", "on", "or", "PM", "so", "to", "tv", "up", "us", "we"],
        3: ["act", "add", "age", "ago", "air", "all", "and", "any", "are", "arm", "art", "ask", "bad", "bag", "bar",
            "bed", "big", "bit", "box", "boy", "but", "buy", "can", "car", "cup", "cut", "day", "die", "dog", "eat",
            "end", "eye", "far", "few", "fly", "for", "gas", "get", "gun", "guy", "has", "her", "him", "his", "hit",
            "hot", "how", "its", "job", "key", "kid", "law", "lay", "leg", "let", "lie", "lot", "low", "man", "may",
            "mrs", "new", "nor", "not", "now", "n't", "off", "oil", "old", "one", "our", "out", "own", "pay", "per",
            "put", "red", "run", "say", "sea", "see", "set", "sex", "she", "sit", "six", "son", "tax", "ten", "the",
            "too", "top", "try", "two", "use", "war", "was", "way", "who", "why", "win", "yes", "yet", "you"],
        4: ["able", "also", "area", "away", "baby", "back", "ball", "bank", "base", "beat", "best", "bill", "blue",
            "body", "book", "born", "both", "call", "card", "care", "case", "cell", "city", "cold", "come", "cost",
            "dark", "data", "dead", "deal", "deep", "door", "down", "draw", "drop", "drug", "each", "east", "easy",
            "edge", "else", "even", "ever", "face", "fact", "fail", "fall", "fast", "fear", "feel", "fill", "film",
            "find", "fine", "fire", "firm", "fish", "five", "food", "foot", "form", "four", "free", "from", "full",
            "fund", "game", "girl", "give", "goal", "good", "grow", "hair", "half", "hand", "hang", "hard", "have",
            "head", "hear", "heat", "help", "here", "high", "hold", "home", "hope", "hour", "huge", "idea", "into",
            "item", "join", "just", "keep", "kill", "kind", "know", "land", "last", "late", "lead", "left", "less",
            "life", "like", "line", "list", "live", "long", "look", "lose", "loss", "love", "main", "make", "many",
            "mean", "meet", "mind", "miss", "more", "most", "move", "much", "must", "name", "near", "need", "news",
            "next", "nice", "none", "note", "once", "only", "onto", "open", "over", "page", "pain", "part", "pass",
            "past", "pick", "plan", "play", "poor", "pull", "push", "race", "rate", "read", "real", "rest", "rich",
            "rise", "risk", "road", "rock", "role", "room", "rule", "safe", "same", "save", "seat", "seek", "seem",
            "sell", "send", "shot", "show", "side", "sign", "sing", "site", "size", "skin", "some", "song", "soon",
            "sort", "star", "stay", "step", "stop", "such", "sure", "take", "talk", "task", "team", "tell", "tend",
            "term", "test", "than", "that", "them", "then", "they", "this", "thus", "time", "town", "tree", "trip",
            "true", "turn", "type", "unit", "upon", "very", "view", "vote", "wait", "walk", "wall", "want", "wear",
            "week", "well", "west", "what", "when", "whom", "wide", "wife", "will", "wind", "wish", "with", "word",
            "work", "yard", "yeah", "year", "your"],
        5: ["about", "above", "admit", "adult", "after", "again", "agent", "agree", "ahead", "allow", "alone", "along",
            "among", "apply", "argue", "avoid", "begin", "black", "blood", "board", "break", "bring", "build", "carry",
            "catch", "cause", "chair", "check", "child", "civil", "claim", "class", "clear", "close", "coach", "color",
            "could", "court", "cover", "crime", "death", "dream", "drive", "early", "eight", "enjoy", "enter", "event",
            "every", "exist", "field", "fight", "final", "first", "floor", "focus", "force", "front", "glass", "great",
            "green", "group", "guess", "happy", "heart", "heavy", "hotel", "house", "human", "image", "issue", "large",
            "later", "laugh", "learn", "least", "leave", "legal", "level", "light", "local", "major", "maybe", "media",
            "might", "model", "money", "month", "mouth", "movie", "music", "never", "night", "north", "occur", "offer",
            "often", "order", "other", "owner", "paper", "party", "peace", "phone", "piece", "place", "plant", "point",
            "power", "price", "prove", "quite", "radio", "raise", "range", "reach", "ready", "right", "scene", "score",
            "sense", "serve", "seven", "shake", "share", "shoot", "short", "since", "skill", "small", "smile", "sound",
            "south", "space", "speak", "spend", "sport", "staff", "stage", "stand", "start", "state", "still", "stock",
            "store", "story", "study", "stuff", "style", "table", "teach", "thank", "their", "there", "these", "thing",
            "think", "third", "those", "three", "throw", "today", "total", "tough", "trade", "treat", "trial", "truth",
            "under", "until", "value", "visit", "voice", "watch", "water", "where", "which", "while", "white", "whole",
            "whose", "woman", "world", "worry", "would", "write", "wrong", "young"],
        6: ["accept", "across", "action", "affect", "agency", "almost", "always", "amount", "animal", "answer",
            "anyone", "appear", "around", "arrive", "artist", "assume", "attack", "author", "become", "before",
            "behind", "better", "beyond", "budget", "camera", "cancer", "career", "center", "chance", "change",
            "charge", "choice", "choose", "church", "common", "couple", "course", "create", "debate", "decade",
            "decide", "degree", "design", "detail", "dinner", "doctor", "during", "effect", "effort", "either",
            "energy", "enough", "entire", "expect", "expert", "factor", "family", "father", "figure", "finger",
            "finish", "follow", "forget", "former", "friend", "future", "garden", "ground", "growth", "happen",
            "health", "impact", "indeed", "inside", "itself", "lawyer", "leader", "letter", "likely", "listen",
            "little", "manage", "market", "matter", "member", "memory", "method", "middle", "minute", "modern",
            "moment", "mother", "myself", "nation", "nature", "nearly", "notice", "number", "office", "option",
            "others", "parent", "people", "period", "person", "player", "police", "policy", "pretty", "public",
            "rather", "really", "reason", "recent", "record", "reduce", "region", "relate", "remain", "remove",
            "report", "result", "return", "reveal", "school", "season", "second", "senior", "series", "sexual",
            "should", "simple", "simply", "single", "sister", "social", "source", "speech", "spring", "street",
            "strong", "suffer", "summer", "system", "theory", "though", "threat", "toward", "travel", "victim",
            "weapon", "weight", "window", "within", "wonder", "worker", "writer"],
        7: ["ability", "account", "address", "against", "already", "another", "article", "because", "believe",
            "benefit", "between", "billion", "brother", "capital", "central", "century", "certain", "citizen",
            "clearly", "college", "company", "compare", "concern", "contain", "control", "country", "culture",
            "current", "defense", "despite", "develop", "discuss", "disease", "economy", "evening", "exactly",
            "example", "explain", "federal", "feeling", "finally", "foreign", "forward", "general", "herself",
            "himself", "history", "however", "hundred", "husband", "imagine", "improve", "include", "instead",
            "involve", "kitchen", "machine", "manager", "measure", "medical", "meeting", "mention", "message",
            "million", "mission", "morning", "natural", "network", "nothing", "officer", "outside", "partner",
            "patient", "pattern", "perform", "perhaps", "picture", "popular", "prepare", "present", "prevent",
            "private", "problem", "process", "produce", "product", "program", "project", "protect", "provide",
            "purpose", "quality", "quickly", "reality", "realize", "receive", "reflect", "require", "respond",
            "science", "section", "serious", "service", "several", "similar", "society", "soldier", "someone",
            "special", "station", "student", "subject", "success", "suggest", "support", "surface", "teacher",
            "thought", "through", "tonight", "trouble", "usually", "various", "western", "whether", "without"],
        8: ["activity", "actually", "although", "American", "analysis", "anything", "approach", "attorney", "audience",
            "behavior", "building", "business", "campaign", "computer", "Congress", "consider", "consumer", "continue",
            "cultural", "customer", "daughter", "decision", "Democrat", "describe", "director", "discover", "economic",
            "election", "employee", "everyone", "evidence", "hospital", "identify", "increase", "indicate", "industry",
            "interest", "language", "magazine", "maintain", "majority", "marriage", "material", "military", "movement",
            "national", "official", "painting", "personal", "physical", "politics", "position", "positive", "possible",
            "practice", "pressure", "probably", "property", "question", "recently", "remember", "research", "resource",
            "response", "security", "shoulder", "somebody", "southern", "specific", "standard", "strategy", "suddenly",
            "thousand", "together", "training", "violence", "whatever", "yourself"],
        9: ["according", "agreement", "attention", "authority", "available", "beautiful", "candidate", "certainly",
            "challenge", "character", "community", "condition", "determine", "different", "difficult", "direction",
            "education", "establish", "everybody", "executive", "financial", "important", "including", "interview",
            "knowledge", "necessary", "newspaper", "operation", "political", "president", "professor", "recognize",
            "religious", "represent", "scientist", "situation", "something", "sometimes", "statement", "structure",
            "treatment"],
        10: ["collection", "commercial", "conference", "democratic", "difference", "discussion", "especially",
             "everything", "experience", "generation", "government", "individual", "investment", "management",
             "particular", "population", "production", "Republican", "successful", "technology", "television",
             "themselves", "throughout", "understand"],
        11: ["development", "environment", "information", "institution", "interesting", "opportunity", "participant",
             "performance", "significant", "traditional"],
        12: ["organization", "particularly", "professional", "relationship"],
        13: ["environmental", "international"],
        14: ["administration", "responsibility"]
    }
}



class LetterCounter:
    def __init__(self, text="", alphabet=[]):
        self.text = text
        self.alphabet = alphabet
        self.counter = {}

    def __repr__(self):
        return 'LetterCounter(text=%s, alphabet=[%s])' % (self.text, {", ".join(str(c) for c in self.alphabet)})

    def __str__(self):
        counter = self.count()
        return '[%s]' % ({", ".join(": ".join([str(c), str(counter[c])]) for c in counter)})

    def count(self):
        if self.counter == {}:
            for letter in self.alphabet:
                self.counter[str(letter).lower()] = 0
            for character in self.text:
                if character.isalnum():
                    standardised_character = str(character).lower()
                    if standardised_character not in self.alphabet:
                        self.alphabet.append(standardised_character)
                        self.counter[standardised_character] = 1
                    else:
                        self.counter[standardised_character] += 1
        return self.counter

    def set_text(self, text):
        self.text = text
        self.counter = {}

    def set_alphabet(self, alphabet):
        self.alphabet = alphabet
        self.counter = {}

    def get_text(self):
        return self.text

    def get_alphabet(self):
        if self.alphabet == [] and len(self.text) > 0:
            for character in self.text:
                if character not in self.alphabet:
                    self.alphabet.append(character)
        return self.alphabet



from .ngram_counter import NgramCounter


class NgramFrequency:
    def __init__(self, nvalue=2, text="", joined=True):
        self.text = text
        self.nvalue = int(nvalue)
        self.joined = joined
        self.frequency = {}
        self.ngram_count = NgramCounter(nvalue=nvalue, text=text, joined=joined)

    def __repr__(self):
        return 'NgramFrequency(nvalue={nvalue}, text={text}, joined={joined})'.format(
            nvalue=self.nvalue,
            text=self.text,
            joined=self.joined
        )

    def __str__(self):
        frequency = self.calculate()
        return '{}'.format({", ".join(": ".join([str(c), str(frequency[c])]) for c in frequency)})

    def calculate(self):
        if self.frequency == {} and len(self.text.strip()) > 0:
            counter = self.ngram_count.count()
            length = 0
            for item in counter.keys():
                length += counter[item]
            for item in counter.keys():
                self.frequency[item] = counter[item] / length
        return self.frequency

    def set_text(self, text):
        self.text = text
        self.frequency = {}
        self.ngram_count = NgramCounter(nvalue=self.nvalue, text=text)

    def get_text(self):
        return self.text



from .index_of_coincidence import IndexOfCoincidence
from .letter_counter import LetterCounter
from .letter_frequency import LetterFrequency
from .ngram_counter import NgramCounter
from .ngram_frequency import NgramFrequency



from ..data.index_of_coincidence_by_language import index_of_coincidence_by_language
from ..stats.index_of_coincidence import IndexOfCoincidence


def _ic_estimate(text):
    probable_language = None
    difference = None
    ic_calculator = IndexOfCoincidence(text)
    ic = ic_calculator.calculate()
    for language in index_of_coincidence_by_language:
        if probable_language is None:
            probable_language = language
            difference = abs(ic - index_of_coincidence_by_language[language])
        else:
            if abs(ic - index_of_coincidence_by_language[language]) < difference:
                probable_language = language
                difference = abs(ic - index_of_coincidence_by_language[language])
    return {
        'language': language,
        'index_of_coincidence': ic,
        'index_of_coincidence_of_language': index_of_coincidence_by_language[language]
    }


METHODS = [
    {
        'id': 'ic',
        'name': 'index of coincidence',
        'estimate': _ic_estimate
    }
]


class LanguageEstimator:
    def __init__(self, text, methods):
        self.text = text
        self.methods = []
        for method in methods:
            if method in METHODS.keys():
                self.methods.append(METHODS[method])
            else:
                raise ValueError("Unknown estimation method: " + method)

    def estimate(self):
        estimates = {}
        for method in self.methods:
            estimates[method['id']] = {
                'id': method['id'],
                'name': method['name'],
                'estimate': method['estimate'](self.text)
            }



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="crypto_tools",
    packages=find_packages(exclude=['tests*']),
    version="1.0.9",
    author="Rocco Barbini",
    author_email="roccobarbi@gmail.com",
    description="Tools that can be used to build software for breaking \"traditional\" cyphers and codes.",
    long_description_content_type="text/markdown; charset=UTF-8; variant=GFM",
    long_description=long_description,
    url="https://github.com/roccobarbi/crypto_tools",
    download_url="https://github.com/roccobarbi/crypto_tools/releases/tag/1.0.9",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



from .letter_counter import LetterCounter


class LetterFrequency:
    def __init__(self, text=""):
        self.text = text
        self.frequency = {}
        self.letter_count = LetterCounter(text=text)

    def __repr__(self):
        return 'LetterFrequency(text=%s)' % self.text

    def __str__(self):
        frequency = self.calculate()
        return '[%s]' % ({", ".join(": ".join([str(c), str(frequency[c])]) for c in frequency)})

    def calculate(self):
        if len(self.text.strip()) == 0:
            return {}
        if self.frequency == {}:
            counter = self.letter_count.count()
            length = 0
            for item in counter.keys():
                length += counter[item]
            for item in counter.keys():
                self.frequency[item] = counter[item] / length
        return self.frequency

    def set_text(self, text):
        self.text = text
        self.frequency = {}
        self.letter_count = LetterCounter(text=text)

    def get_text(self):
        return self.text

    def get_alphabet(self):
        return self.letter_count.get_alphabet()